@@ -52,7 +52,7 @@ G_BEGIN_DECLS
  *
  * The typelib has the following general format.
  *
- * typelib ::= header, directory, blobs, attributes, attributedata
+ * typelib ::= header, section-index, directory, blobs, attributes, attributedata
  *
  * directory ::= list of entries
  *
@@ -72,6 +72,11 @@ G_BEGIN_DECLS
 /*
 TYPELIB HISTORY
 -----
+
+Version 1.1
+- Add ref/unref/set-value/get-value functions to Object, to be able
+  to support instantiatable fundamental types which are not GObject based.
+
 Version 1.0
 - Rename class_struct to gtype_struct, add to interfaces
 
@@ -176,6 +181,7 @@ typedef enum {
         ((blob)->blob_type == BLOB_TYPE_STRUCT ||   \
          (blob)->blob_type == BLOB_TYPE_UNION  ||   \
          (blob)->blob_type == BLOB_TYPE_ENUM   ||   \
+         (blob)->blob_type == BLOB_TYPE_FLAGS  ||   \
          (blob)->blob_type == BLOB_TYPE_OBJECT ||   \
          (blob)->blob_type == BLOB_TYPE_INTERFACE)
 
@@ -227,6 +233,7 @@ typedef enum {
  * write parser which continue to work if the format is extended by
  * adding new fields before the first flexible array member in
  * variable-size blobs.
+ * @sections: Offset of section blob array
  *
  * The header structure appears exactly once at the beginning of a typelib.  It is a
  * collection of meta-information, such as the number of entries and dependencies.
@@ -272,10 +279,34 @@ typedef struct {
   guint16 interface_blob_size;
   guint16 union_blob_size;
 
+  guint32 sections;
+
   /* <private> */
-  guint16 padding[7];
+  guint16 padding[5];
 } Header;
 
+typedef enum {
+  GI_SECTION_END = 0,
+  GI_SECTION_DIRECTORY_INDEX = 1
+} SectionType;
+
+/**
+ * Section:
+ * @id: A #SectionType
+ * @offset: Integer offset for this section
+ *
+ * A section is a blob of data that's (at least theoretically) optional,
+ * and may or may not be present in the typelib.  Presently, just used
+ * for the directory index.  This allows a form of dynamic extensibility
+ * with different tradeoffs from the format minor version.
+ *
+ */
+typedef struct {
+  guint32 id;
+  guint32 offset;
+} Section;
+
+
 /**
  * DirEntry:
  * @blob_type: A #GTypelibBlobType
@@ -626,6 +657,7 @@ typedef struct {
 /**
  * ValueBlob:
  * @deprecated: Whether this value is deprecated
+ * @unsigned_value: if set, value is a 32-bit unsigned integer cast to gint32
  * @value: The numerical value
  * @name: Name of blob
  *
@@ -633,8 +665,9 @@ typedef struct {
  */
 typedef struct {
   guint32 deprecated : 1;
+  guint32 unsigned_value : 1;
   /* <private> */
-  guint32 reserved   :31;
+  guint32 reserved   :30;
   /* <public> */
   guint32 name;
   gint32 value;
@@ -918,6 +951,8 @@ typedef struct {
 /**
  * ObjectBlob:
  * @blob_type: #BLOB_TYPE_OBJECT
+ * @fundamental: this object is not a GObject derived type, instead it's
+ * an additional fundamental type.
  * @gtype_name: String name of the associated #GType
  * @gtype_init: String naming the symbol which gets the runtime #GType
  * @parent: The directory index of the parent type. This is only set for
@@ -938,12 +973,21 @@ typedef struct {
  * @signals: Describes the signals.
  * @vfuncs: Describes the virtual functions.
  * @constants: Describes the constants.
+ * @ref_func: String pointing to a function which can be called to increase
+ * the reference count for an instance of this object type.
+ * @unref_func: String pointing to a function which can be called to decrease
+ * the reference count for an instance of this object type.
+ * @set_value_func: String pointing to a function which can be called to
+ * convert a pointer of this object to a GValue
+ * @get_value_func: String pointing to a function which can be called to
+ * convert extract a pointer to this object from a GValue
  */
 typedef struct {
   guint16   blob_type;  /* 7 */
   guint16   deprecated   : 1;
   guint16   abstract     : 1;
-  guint16   reserved     :14;
+  guint16   fundamental  : 1;
+  guint16   reserved     :13;
   guint32   name;
 
   guint32   gtype_name;
@@ -961,6 +1005,11 @@ typedef struct {
   guint16   n_constants;
   guint16   reserved2;
 
+  guint32   ref_func;
+  guint32   unref_func;
+  guint32   set_value_func;
+  guint32   get_value_func;
+
   guint32   reserved3;
   guint32   reserved4;
 
@@ -1063,9 +1112,9 @@ typedef struct {
 } AttributeBlob;
 
 /**
- * GTypelib:
+ * GITypelib:
  */
-struct _GTypelib {
+struct _GITypelib {
   /* <private> */
   guchar *data;
   gsize len;
@@ -1075,23 +1124,30 @@ struct _GTypelib {
   gboolean open_attempted;
 };
 
-DirEntry *g_typelib_get_dir_entry (GTypelib *typelib,
+DirEntry *g_typelib_get_dir_entry (GITypelib *typelib,
 				   guint16   index);
 
+DirEntry *g_typelib_get_dir_entry_by_name (GITypelib *typelib,
+					   const char *name);
+
+DirEntry *g_typelib_get_dir_entry_by_gtype (GITypelib *typelib,
+					    gboolean   fastpass,
+					    GType      gtype);
+
 void      g_typelib_check_sanity (void);
 
 #define   g_typelib_get_string(typelib,offset) ((const gchar*)&(typelib->data)[(offset)])
 
 
 /**
- * GTypelibError:
+ * GITypelibError:
  * @G_TYPELIB_ERROR_INVALID: the typelib is invalid
  * @G_TYPELIB_ERROR_INVALID_HEADER: the typelib header is invalid
  * @G_TYPELIB_ERROR_INVALID_DIRECTORY: the typelib directory is invalid
  * @G_TYPELIB_ERROR_INVALID_ENTRY: a typelib entry is invalid
  * @G_TYPELIB_ERROR_INVALID_BLOB: a typelib blob is invalid
  *
- * A error set while validating the #GTypelib
+ * A error set while validating the #GITypelib
  */
 typedef enum
 {
@@ -1100,13 +1156,13 @@ typedef enum
   G_TYPELIB_ERROR_INVALID_DIRECTORY,
   G_TYPELIB_ERROR_INVALID_ENTRY,
   G_TYPELIB_ERROR_INVALID_BLOB
-} GTypelibError;
+} GITypelibError;
 
 #define G_TYPELIB_ERROR (g_typelib_error_quark ())
 
 GQuark g_typelib_error_quark (void);
 
-gboolean g_typelib_validate (GTypelib  *typelib,
+gboolean g_typelib_validate (GITypelib  *typelib,
 			     GError    **error);
 
 
@@ -1114,6 +1170,23 @@ gboolean g_typelib_validate (GTypelib  *typelib,
 AttributeBlob *_attribute_blob_find_first (GIBaseInfo *info,
                                            guint32     blob_offset);
 
+typedef struct _GITypelibHashBuilder GITypelibHashBuilder;
+
+GITypelibHashBuilder * _gi_typelib_hash_builder_new (void);
+
+void _gi_typelib_hash_builder_add_string (GITypelibHashBuilder *builder, const char *str, guint16 value);
+
+gboolean _gi_typelib_hash_builder_prepare (GITypelibHashBuilder *builder);
+
+guint32 _gi_typelib_hash_builder_get_buffer_size (GITypelibHashBuilder *builder);
+
+void _gi_typelib_hash_builder_pack (GITypelibHashBuilder *builder, guint8* mem, guint32 size);
+
+void _gi_typelib_hash_builder_destroy (GITypelibHashBuilder *builder);
+
+guint16 _gi_typelib_hash_search (guint8* memory, const char *str);
+
+
 G_END_DECLS
 
 #endif  /* __G_TYPELIB_H__ */
